<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 90be9e89b02f $
 * $Author: Lukas Imhof $
 * $Date: 2020-06-15 16:02 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Basic view class for customer filters
 *
 */
class we_customer_filterView{
	/** @var we_customer_abstractFilter filter for view	 */
	protected $filter = null;

	/** @var int width of filter	 */
	protected $width = 0;

	/** @var string */
	protected $viewType = 'base';

	/** @var we_contents_root */
	var $we_doc;

	/**
	 * Constructor
	 *
	 * @param we_customer_abstractFilter $filter
	 * @param integer $width
	 */
	function __construct(we_customer_abstractFilter &$filter, int $width = 0){
		$this->setFilter($filter);
		$this->setWidth($width);
	}

	/**
	 * Gets the HTML and Javascript for the filter
	 *
	 * @return string
	 */
	function getFilterHTML(bool $ShowModeNone = false): string{
		$mode = $this->filter->getMode();

		// Radio buttons
		$modeRadioOff = we_html_forms::radiobuttonV(strval(we_customer_abstractFilter::OFF), (string) $mode, 'wecf_mode', g_l('modules_customerFilter', '[mode_off]'), true, '', "we_cmd('setHot');updateView();");
		$modeRadioNone = ($ShowModeNone ?
			we_html_forms::radiobuttonV(strval(we_customer_abstractFilter::NOT_LOGGED_IN_USERS), (string) $mode, 'wecf_mode', g_l('modules_customerFilter', '[mode_none]'), true, '', "we_cmd('setHot');updateView();") :
			'');

		$modeRadioAll = we_html_forms::radiobuttonV(strval(we_customer_abstractFilter::ALL), (string) $mode, 'wecf_mode', g_l('modules_customerFilter', '[mode_all]'), true, '', "we_cmd('setHot');updateView();");
		$modeRadioSpecific = we_html_forms::radiobuttonV(strval(we_customer_abstractFilter::SPECIFIC), (string) $mode, 'wecf_mode', g_l('modules_customerFilter', '[mode_specific]'), true, '', "we_cmd('setHot');updateView();");
		$modeRadioFilter = we_html_forms::radiobuttonV(strval(we_customer_abstractFilter::FILTER), (string) $mode, 'wecf_mode', g_l('modules_customerFilter', '[mode_filter]'), true, '', "we_cmd('setHot');updateView();");

		$filters = [
			'specificCustomersEdit' => [],
			'blackListEdit' => [],
			'whiteListEdit' => [],
		];
		// Selector for specific customers
		list($specificCustomersSelect, $filters['specificCustomersEdit'] ) = $this->getMultiEdit('specificCustomersEdit', $this->filter->getSpecificCustomers(), '', $mode === we_customer_abstractFilter::SPECIFIC);
		// Selector blacklist

		list($blackListSelect, $filters['blackListEdit']) = $this->getMultiEdit('blackListEdit', $this->filter->getBlackList(), g_l('modules_customerFilter', '[black_list]'), $mode === we_customer_abstractFilter::FILTER);
		// Selector for whitelist

		list($whiteListSelect, $filters['whiteListEdit']) = $this->getMultiEdit('whiteListEdit', $this->filter->getWhiteList(), g_l('modules_customerFilter', '[white_list]'), $mode === we_customer_abstractFilter::FILTER);
		// customer filter

		$filterCustomers = $this->getDiv($this->getHTMLCustomerFilter(), 'filterCustomerDiv', $mode === we_customer_abstractFilter::FILTER, 25);


		// concate and output

		return
			$modeRadioOff . $modeRadioAll . $modeRadioSpecific . $specificCustomersSelect . $modeRadioFilter . $filterCustomers . $blackListSelect . $whiteListSelect . $modeRadioNone .
			we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/customer_filterLogic.js', 'initFilters();', [
				'id' => 'loadcfilter',
				'data-cfilter' => setDynamicVar([
					'delButton' => we_html_button::create_button(we_html_button::TRASH, "javascript:" . we_base_constants::PLACEHOLDER . ";we_cmd('setHot');"),
					'type' => $this->viewType,
					'filterWidth' => $this->width,
					'filters' => $filters
		])]);
	}

	public function getFilterCustomers(): string{
		$this->filter->setMode(we_customer_abstractFilter::FILTER);

		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/customer_filterLogic.js', '', [
				'id' => 'loadcfilter',
				'data-cfilter' => setDynamicVar([
					'type' => $this->viewType
			])]) .
			$this->getDiv($this->getHTMLCustomerFilter(true), 'filterCustomerDiv', true);
	}

	/**
	 * Creates a HTML div
	 *
	 * @param string $content  Content of the div
	 * @param string $divId id of the div
	 * @param boolean $isVisible
	 * @param integer $marginLeft
	 * @static
	 * @return string
	 */
	function getDiv(string $content = '', string $divId = '', bool $isVisible = true, int $marginLeft = 0): string{
		return we_html_element::div([
				'id' => ($divId ?: ''),
				'style' => 'display:' . ($isVisible ? 'block' : 'none') . ';margin-left:' . $marginLeft . 'px;'
				], $content);
	}

	/**
	 * Creates a multi edit gui element for use in customerFilterViews
	 *
	 * @param string $name
	 * @param int[] $data
	 * @param string $headline
	 * @param boolean $isVisible
	 * @return array
	 */
	private function getMultiEdit(string $name, array $data, string $headline = '', bool $isVisible = true): array{
		static $settingsSQL = false;
		if(!$settingsSQL){
			$settings = new we_customer_settings();
			$settings->load();
			$settingsSQL = $settings->treeTextFormatSQL;
		}

		$items = ($data ? DB_WE::$inst->getAllq('SELECT ID,' . $settingsSQL . ' AS Text FROM ' . CUSTOMER_TABLE . ' WHERE ID IN (' . implode(',', $data) . ') ORDER BY Text', false) : [
			]);

		$buttonTable = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:we_cmd('multiedit_delAll','customer','" . $name . "');") . we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_customer_selector',{jsCmd:'multiedit_addItem,customer,multiple:true});");

		$select = ($headline ? '<div class="defaultfont">' . $headline . '</div>' : '') .
			'<div id="' . $name . 'MultiEdit" style="overflow:auto;background-color:white;padding:5px;width:' . $this->width . 'px; height: 120px; border: #AAAAAA solid 1px;margin-bottom:5px;"></div>' .
			'<div style="width:' . ($this->width + 13) . 'px;text-align:right">' . $buttonTable . '</div>';
		return [we_html_element::hiddens([$name . 'Control' => we_base_request::string($name . 'Control'),]) . $this->getDiv($select, $name . 'Div', $isVisible, 22),
			$items];
	}

	private function getHTMLCustomerFilter(bool $startEmpty = false): string{
		$dbData = array_column(DB_WE::$inst->getAllq('SHOW FIELDS FROM ' . CUSTOMER_TABLE), 'Field');

		$filter_args = we_html_tools::groupArray(array_combine($dbData, $dbData));
		$filter_op = [
			we_customer_abstractFilter::OP_EQ => g_l('modules_customerFilter', '[equal]'),
			we_customer_abstractFilter::OP_NEQ => g_l('modules_customerFilter', '[not_equal]'),
			we_customer_abstractFilter::OP_LESS => g_l('modules_customerFilter', '[less]'),
			we_customer_abstractFilter::OP_LEQ => g_l('modules_customerFilter', '[less_equal]'),
			we_customer_abstractFilter::OP_GREATER => g_l('modules_customerFilter', '[greater]'),
			we_customer_abstractFilter::OP_GEQ => g_l('modules_customerFilter', '[greater_equal]'),
			we_customer_abstractFilter::OP_STARTS_WITH => g_l('modules_customerFilter', '[starts_with]'),
			we_customer_abstractFilter::OP_ENDS_WITH => g_l('modules_customerFilter', '[ends_with]'),
			we_customer_abstractFilter::OP_CONTAINS => g_l('modules_customerFilter', '[contains]'),
			we_customer_abstractFilter::OP_NOT_CONTAINS => g_l('modules_customerFilter', '[not_contains]'),
			we_customer_abstractFilter::OP_IN => g_l('modules_customerFilter', '[in]'),
			we_customer_abstractFilter::OP_NOT_IN => g_l('modules_customerFilter', '[not_in]'),
		];

		$filter_logic = [
			'AND' => g_l('modules_customerFilter', '[AND]'),
			'OR' => g_l('modules_customerFilter', '[OR]')
		];

		$filter = $this->filter->getFilter();

		if(!$startEmpty && empty($filter)){
			$filter = [
				[
					'logic' => '',
					'field' => 'id',
					'operation' => 0,
					'value' => ''
				]
			];
			$this->filter->setFilter($filter);
		}

		$i = 0;
		$adv_row = '';

		foreach($filter as $key => $value){
			if(!is_array($value) || empty($value)){
				continue;
			}
			$value['logic'] = trim($value['logic']);
			$adv_row .= '
<tr id="filterRow_' . $i . '">
	<td style="padding-top: ' . ($value['logic'] === "OR" ? "10px;border-top:1px solid grey" : "4px;border-top:0") . ';padding-bottom:' .
				((isset($filter[$key + 1]) && $filter[$key + 1]['logic'] === 'OR') ? '10px' : '0px') . ';width:64px;">' .
				(($i == 0) ? '' : we_html_tools::select('filterLogic_' . $i, $filter_logic, 1, $value['logic'], false, ['onchange' => "wecf_logic_changed(this);", 'class' => "defaultfont logicFilterInput"])) . '</td>
	<td style="padding-top: ' . ($value['logic'] === "OR" ? "10px;border-top:1px solid grey" : "4px;border-top:0") . ';padding-bottom:' .
				((isset($filter[$key + 1]) && $filter[$key + 1]['logic'] === 'OR') ? '10px' : '0px') . ';">' .
				we_html_tools::select('filterSelect_' . $i, $filter_args, 1, $value['field'], false, ['onchange' => "we_cmd('setHot');", 'class' => "defaultfont leftFilterInput"]) . '</td>
	<td style="padding-top: ' . ($value['logic'] === 'OR' ? "10px;border-top:1px solid grey" : "4px;border-top:0") . ';padding-bottom:' .
				((isset($filter[$key + 1]) && $filter[$key + 1]['logic'] === 'OR') ? '10px' : '0px') . ';">' .
				we_html_tools::select('filterOperation_' . $i, $filter_op, 1, $value['operation'], false, ['onchange' => "we_cmd('setHot');", 'class' => "defaultfont middleFilterInput"]) . '</td>
	<td style="padding-top: ' . ($value['logic'] === 'OR' ? "10px;border-top:1px solid grey" : "4px;border-top:0") . ';padding-bottom:' .
				((isset($filter[$key + 1]) && $filter[$key + 1]['logic'] === 'OR') ? '10px' : '0px') . ';">' .
				'<input name="filterValue_' . $i . '" value="' . $value['value'] . '" type="text" onchange="we_cmd(\'setHot\');" class="defaultfont rightFilterInput"/></td>
	<td style="padding-top: ' . ($value['logic'] === 'OR' ? "10px;border-top:1px solid grey" : "4px;border-top:0") . ';padding-bottom:' .
				((isset($filter[$key + 1]) && $filter[$key + 1]['logic'] === 'OR') ? '10px' : '0px') . ';">' .
				we_html_button::create_button(we_html_button::PLUS, "javascript:addRow(" . ($i + 1) . ")") . '</td>
	<td style="padding-left:5px;padding-top: ' . ($value['logic'] === "OR" ? "10px;border-top:1px solid grey" : "4px;border-top:0") . ';padding-bottom:' .
				((isset($filter[$key + 1]) && $filter[$key + 1]['logic'] === 'OR') ? '10px' : '0px') . ';width:25px;">' .
				(($i != 0 || $startEmpty) ? we_html_button::create_button(we_html_button::TRASH, "javascript:delRow($i)") : '') . '</td>
</tr>';
			$i++;
		}

		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/customer_filter.js', '', ['id' => 'loadVarCustomerFilter', 'data-customerFilter' => setDynamicVar([
					'filter' => [
						'logic' => we_html_tools::select('', $filter_logic),
						'args' => we_html_tools::select('', $filter_args),
						'op' => we_html_tools::select('', $filter_op),
						'startEmpty' => $startEmpty,
					],
					'buttons' => [
						'add' => we_html_button::create_button(we_html_button::PLUS, "javascript:addRow(__CNT__)"),
						'trash' => we_html_button::create_button(we_html_button::TRASH, "javascript:delRow(__CNT__)"),
					]
			])]) . '
<table class="default" style="width:' . $this->width . 'px;height:50px;">
	<tbody id="filterTable">
		' . $adv_row . '
	</tbody>
</table>' .
			($filter ? '' : '<div>' . we_html_button::create_button(we_html_button::PLUS, "javascript:addRow();") . '</div>');
	}

	/* mutator and accessor methods */

	/**
	 * accessor method for $this->_filter
	 *
	 * @return we_customer_abstractFilter
	 */
	function getFilter(): we_customer_abstractFilter{
		return $this->filter;
	}

	/**
	 * mutator method for $this->_filter
	 *
	 * @param we_customer_abstractFilter $filter
	 */
	function setFilter(we_customer_abstractFilter &$filter): void{
		$this->filter = $filter;
	}

	/**
	 * accessor method for $this->_width
	 *
	 * @return integer
	 */
	function getWidth(): int{
		return $this->width;
	}

	/**
	 * mutator method for $this->_width
	 *
	 * @param integer $width
	 */
	function setWidth(int $width): void{
		$this->width = $width;
	}

}
