<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 938a3571c9bf $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-03-19 12:37 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_html_tbody extends we_html_baseCollection{
	/** @var int */
	private $cols = 0;

	/**
	 * @param ?we_html_tr[] $content
	 * @param array<string,string> $attribs
	 */
	public function __construct(int $rows_num = 0, int $cols_num = 0, array $content = null, array $attribs = []){
		parent::__construct('tbody', true, $attribs);
		if($rows_num){
			$this->addRows($rows_num);
			$this->addCol($cols_num);
		} else {
			$this->cols = $cols_num;
		}
		if($content){
			$this->setTableContent($content);
		}
	}

	/**
	 *
	 * @param we_html_tr[] $content
	 * @return void
	 */
	public function setTableContent(array $content): void{
		if(!$content){
			return;
		}
		$this->childs = array_merge($this->childs, $content);
	}

	public function addRow(we_html_tr $row): self{
		if($this->cols != count($row->childs)){
			t_e('row childs does not match', $this->cols, $row);
		}
		$this->childs[] = $row;
		return $this;
	}

	/**
	 * This function adds a row to the table
	 * @param      $rows_num                               int                 (optional)
	 * @see        we_html_table()
	 * @return     integer
	 */
	public function addRows(int $rows_num = 1, array $attribs = []): int{
		$ret = count($this->childs);
		for($i = 0; $i < $rows_num; $i++){
			$childs = [];
			for($j = 0; $j < $this->cols; $j++){
				$childs[] = new we_html_td();
			}
			$this->childs[] = new we_html_tr($childs, $attribs);
		}
		return $ret;
	}

	/**
	 * This function adds a column to the table
	 *
	 * @param      $cols_num                               int                 (optional)
	 * @see        we_html_table()
	 * @return     we_html_table
	 */
	public function addCol(int $cols_num = 1): self{
		$this->cols += $cols_num;
		for($i = 0; $i < $cols_num; $i++){
			foreach($this->childs as &$v){
				$v->childs[] = new we_html_td();
			}
		}

		return $this;
	}

	/**
	 * This function sets the current column being edited
	 *
	 * @param      $rowid                                  int
	 * @param      $colid                                  int
	 * @param      $content                                string              (optional)
	 * @param      $attribs                                array<string,string|string[]>               (optional)
	 *
	 * @return     we_html_table
	 */
	public function setCol(int $rowid, int $colid, string $content, array $attribs = []): self{
		while(!isset($this->childs[$rowid])){
			$this->addRows();
		}
		$row = $this->getChild($rowid);
		'@phan-var we_html_baseCollection $row';
		$col = $row->getChild($colid);
		if(!$col){
			t_e('not enough cols defined (' . $this->cols . ')');
			while(!$col){
				$this->addCol();
				$col = $row->getChild($colid);
			}
		}
		$col->setAttributes($attribs);
		if($attribs && ($cspan = $col->getAttribute('colspan')) && --$cspan){
			for($pos = 1; $pos <= $cspan; $pos++){
				unset($row->childs[$colid + $pos]);
			}
		}
		$col->content = $content;
		return $this;
	}

	/**
	 * Assigns the attributes of a column
	 *
	 * @param      int $rowid
	 * @param      array<string,string> $attribs                                (optional)
	 *
	 * @return     we_html_table
	 */
	public function setRowAttributes(int $rowid, array $attribs = []): self{
		$row = & $this->getChild($rowid);
		$row->setAttributes($attribs);
		return $this;
	}

	/**
	 * Assigns the attributes of a column
	 *
	 * @param      $rowid                                  int
	 * @param      $colid                                  int
	 * @param      $attribs                                array               (optional)
	 *
	 * @return     we_html_table
	 */
	public function setColAttributes(int $rowid, int $colid, array $attribs = []): self{
		$row = $this->getChild($rowid);
		'@phan-var we_html_baseCollection $row';
		$col = $row->getChild($colid);
		$col->setAttributes($attribs);
		return $this;
	}

}
