<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 6ce7a65323de $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-22 13:42 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_rebuild_base{
	const TYPE_DOCUMENT = 'document';
	const TYPE_TEMPLATE = 'template';
	const TYPE_OBJECT = 'object';
	const TYPE_MEDIALINKS = 'medialink';

	public static function rebuild(array $data, bool $printIt = false): bool{
		we_base_WE::$rebuildActive = true;
		switch($data['type']){
			case 'thumbnail':
				$imgdoc = new we_document_image($data['id']);
				if($printIt){
					self::print('Rebuilding thumb for image: ' . $imgdoc->Path);
				}

				$imgdoc->Thumbs = $data['thumbs'] ?: [];
				$imgdoc->DocChanged = true;
				$imgdoc->we_save(true);
				unset($imgdoc);
				if($printIt){
					self::print("   done");
				}
				break;
			case 'metadata':
				$imgdoc = new we_document_image($data['id']);
				if($printIt){
					self::print('Rebulding meta data for image: ' . $imgdoc->Path);
				}

				$imgdoc->importMetaData($data['metaFields'], $data['onlyEmpty']);

				$imgdoc->we_save(true);
				unset($imgdoc);
				if($printIt){
					self::print("   done");
				}
				break;
			case self::TYPE_MEDIALINKS:
				switch($data['ct']){
					case we_base_ContentTypes::CATEGORY:
					case we_base_ContentTypes::CUSTOMER:
					case we_base_ContentTypes::GLOSSAR:
					case we_base_ContentTypes::NAVIGATION:
					case we_base_ContentTypes::NEWSLETTER:
						$model = we_base_ContentTypes::getObject($data['ct'], intval($data['id']));
						assert($model instanceof we_category_category || $model instanceof we_customer_customer || $model instanceof we_glossary_glossary || $model instanceof we_navigation_navigation || $model instanceof we_newsletter_newsletter);
						if($printIt){
							self::print('Rebulding Media-Links for: ' . $model->Text);
						}
						$model->registerMediaLinks();
						break;
					default:
						$doc = we_base_ContentTypes::getObject($data['ct']);
						assert($doc instanceof we_contents_root);
						$doc->initByID((int) $data['id'], $doc->Table, empty($data['tt']) ? we_contents_base::LOAD_MAIN_DB : we_contents_base::LOAD_TEMP_DB);
						if($printIt){
							self::print('Rebulding Media-Links for: ' . $doc->Path);
						}
						$doc->correctFields();
						$doc->parseTextareaFields('main');
						$doc->registerMediaLinks();
						unset($doc);
				}
				if($printIt){
					self::print('   done');
				}
				break;
			default:
				switch($data['type']){
					case self::TYPE_DOCUMENT:
						$table = FILE_TABLE;
						break;
					case self::TYPE_TEMPLATE:
						$table = TEMPLATES_TABLE;
						break;
					case self::TYPE_OBJECT:
						$table = OBJECT_FILES_TABLE;
						break;
					default:
						return false;
				}
				$tmp = $data['ct'];
				if(!$tmp){
					t_e('Element ' . $data['id'] . ' in ' . $table . ' has no contenttype');
					return false;
				}
				//unset these, so the main doc is set to the correct doc.
				we_base_WE::resetMainDoc();
				//FIXME: if tags use current set doc, we don't need to set it global
				$we_doc = we_base_ContentTypes::getObject($tmp);
				if($we_doc instanceof we_document_webEdition){//we need this only for tags (static rebuild)
					we_base_WE::setWeDoc($we_doc);
				}
				assert($we_doc instanceof we_document_base || $we_doc instanceof we_contents_template || $we_doc instanceof we_folder_base);
				$we_doc->initByID($data['id'], $table, ($table === OBJECT_FILES_TABLE ? we_contents_base::LOAD_TEMP_DB : we_contents_base::LOAD_MAIN_DB));
				if($printIt){
					self::print('Rebuilding: ' . $we_doc->Path);
				}

				if($we_doc instanceof we_contents_template){
					try{
						$we_doc->Path = $we_doc->constructPath();
					} catch (we_exception_base $e){
						t_e('error writing ' . $we_doc->Path, $e, $we_doc);
					}

					$ret = $we_doc->we_save(true);
					if(!$ret){
						t_e('error writing ' . $we_doc->Path);
					}
				}
				if($data['it']){
					$we_doc->insertAtIndex();
				} else {
					if($we_doc->we_rewrite()){
						$we_doc->we_republish(false);
					} else {
						t_e('error in writing document', $we_doc->Path);
					}
				}
				if($printIt){
					self::print("   done");
				}
		}
		return true;
	}

	private static function print($str): void{
		static $newLine = '';
		if(!$newLine){
			$newLine = count($_SERVER['argv']) ? "\n" : "<br/>\n";
		}
		echo $str . $newLine;
	}

	/**
	 * Create and returns data Array with IDs and other information for the fragmment class for rebuilding documents
	 *
	 * @return array
	 * @param string $btype "rebuild_all" or "rebuild_filter"
	 * @param int[] $categories csv value of category IDs
	 * @param boolean $catAnd true if AND should be used for more than one categories (default=OR)
	 * @param int[] $doctypes csv value of doctypeIDs
	 * @param int[] $folders csv value of directory IDs
	 * @param int $templateID ID of a template (All documents of this template should be rebuilded)
	 */
	public static function getDocuments(string $btype = 'rebuild_all', array $categories = [], bool $catAnd = false, array $doctypes = [], array $folders = [], int $templateID = 0): array{
		switch($btype){
			case we_rebuild_wizard::REBUILD_BTYPE_ALL:
				return self::getAllDocuments();
			case we_rebuild_wizard::REBUILD_BTYPE_TEMPLATES:
				return self::getTemplates();
			case we_rebuild_wizard::REBUILD_BTYPE_DYNAMIC:
				return self::getDynamicDocuments();
			case we_rebuild_wizard::REBUILD_BTYPE_FILTER:
				return self::getFilteredDocuments($categories, $catAnd, $doctypes, $folders, $templateID);
			default:
				return [];
		}
	}

	/**
	 * Create and returns data Array with IDs and other information for the fragmment class for rebuilding all documents and templates (Called from getDocuments())
	 *
	 * @return array
	 */
	private static function getAllDocuments(): array{
		if(!we_base_permission::hasPerm([we_base_perm::REBUILD_ALL])){
			return [];
		}
		return
			array_merge(
			self::getTemplates(true),
											DB_WE::$inst->getAllq('SELECT ID AS id,"' . self::TYPE_DOCUMENT . '" AS `type`,ContentType AS ct,Path AS path,0 AS it FROM ' . FILE_TABLE . ' WHERE IsFolder=1 OR IsPublished=1 ORDER BY IsFolder DESC, LENGTH(Path)'),
																 DB_WE::$inst->getAllq('SELECT ID AS id,"' . self::TYPE_DOCUMENT . '" AS `type`,ContentType AS ct,Path AS path, 0 AS it FROM ' . FILE_TABLE . ' WHERE IsDynamic=0 AND IsPublished=1 AND ContentType="' . we_base_ContentTypes::WEDOCUMENT . '" ORDER BY ID')
		);
	}

	/**
	 * Create and returns data Array with IDs and other information for the fragmment class for rebuilding metadata
	 *
	 * @return array
	 * @param array|string $metaFields array which meta fields should be rebuilt
	 * @param boolean $onlyEmpty if this is true, only empty fields will be imported
	 * @param array|string $metaFolders array with folder Ids
	 */
	public static function getMetadata($metaFields, bool $onlyEmpty, $metaFolders): array{

		$metaFolders = is_array($metaFolders) ? $metaFolders : array_filter(explode(',', $metaFolders));
		$data = [];
		if(we_base_permission::hasPerm([we_base_perm::REBUILD_META])){
			$foldersQuery = count($metaFolders) ? ' AND ParentId IN(' . implode(',', $metaFolders) . ') ' : '';
			DB_WE::$inst->query('SELECT ID,path FROM ' . FILE_TABLE . ' WHERE ContentType="' . we_base_ContentTypes::IMAGE . '" AND Extension IN ("' . implode('","', we_base_ContentTypes::getExtension(we_base_ContentTypes::IMAGE)) . '") ' . $foldersQuery);
			while(DB_WE::$inst->next_record()){
				$data[] = [
					'id' => DB_WE::$inst->f('ID'),
					'type' => 'metadata',
					'onlyEmpty' => $onlyEmpty,
					'path' => DB_WE::$inst->f('path'),
					'metaFields' => $metaFields
				];
			}
		}
		return $data;
	}

	public static function getMediaLinks(): array{
		// delete all media links
		DB_WE::$inst->query('DELETE FROM ' . FILELINK_TABLE . ' WHERE type="media"');


		//FIXME: add permission
		//FIXME: add classes and templates
		$data = DB_WE::$inst->getAllq('SELECT ID AS id,"' . self::TYPE_MEDIALINKS . '" AS type,ContentType AS ct,(IsPublished=0||Publish_Date!=Mod_Date) AS tt,Path AS path,0 AS it FROM ' . FILE_TABLE . ' WHERE
			ContentType IN ("' . we_base_ContentTypes::WEDOCUMENT . '","' . we_base_ContentTypes::IMAGE . '","' . we_base_ContentTypes::CSS . '","' . we_base_ContentTypes::JS . '")
			AND IsFolder=0 ORDER BY ID');

		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$data = array_merge($data, DB_WE::$inst->getAllq('SELECT ID AS id,"' . self::TYPE_MEDIALINKS . '" AS type,ContentType AS ct,"' . we_base_ContentTypes::OBJECT_FILE . '" AS ct,(IsPublished=0||Publish_Date!=Mod_Date) AS tt,Path AS path,0 AS it FROM ' . OBJECT_FILES_TABLE . ' WHERE IsFolder=0 ORDER BY ID')
			);
		}

		$tables = [
			[TEMPLATES_TABLE, 'WHERE ContentType="' . we_base_ContentTypes::TEMPLATE . '"', we_base_ContentTypes::TEMPLATE],
			[(we_modules_info::isActive(we_modules_info::OBJECT) ? OBJECT_TABLE : false), 'WHERE ContentType="' . we_base_ContentTypes::OBJECT . '"', we_base_ContentTypes::OBJECT_FILE],
			[(we_modules_info::isActive(we_modules_info::COLLECTION) ? VFILE_TABLE : false), 'WHERE ContentType="' . we_base_ContentTypes::COLLECTION . '"', we_base_ContentTypes::COLLECTION],
			[CATEGORY_TABLE, 'WHERE Description!=""', we_base_ContentTypes::CATEGORY],
			[(we_modules_info::isActive(we_modules_info::GLOSSARY) ? GLOSSARY_TABLE : false), 'WHERE IsFolder=0 AND type="link"', we_base_ContentTypes::GLOSSAR],
			[NAVIGATION_TABLE, 'WHERE IconID!=0 OR (SelectionType="' . we_navigation_navigation::STYPE_DOCLINK . '" AND LinkID!=0)', we_base_ContentTypes::NAVIGATION],
			[(we_modules_info::isActive(we_modules_info::NEWSLETTER) ? NEWSLETTER_TABLE : false), 'WHERE IsFolder=0', we_base_ContentTypes::NEWSLETTER],
			[(we_modules_info::isActive(we_modules_info::CUSTOMER) ? CUSTOMER_TABLE : false), 'WHERE LoginDenied=0', we_base_ContentTypes::CUSTOMER],
		];

		foreach($tables as $table){
			if($table[0]){
				switch($table[0]){
					case CUSTOMER_TABLE:
						$fields = '"" AS path';
						break;
					default:
						$fields = 'Path AS path';
				}
				$data = array_merge($data, DB_WE::$inst->getAllq('SELECT ID AS id,' . $fields . ',"' . self::TYPE_MEDIALINKS . '" AS type,"' . $table[2] . '" AS ct,0 AS it FROM ' . $table[0] . ' ' . $table[1] . ' ORDER BY ID'));
			}
		}

		return $data;
	}

	/**
	 * @param array<int,string> $all
	 * @param array<int,array{id:int,type:string,ct:string,path:string,it:int}> $data
	 * @return void
	 */
	private static function insertTemplateFoldersInArray(array $all, array &$data): void{
		foreach($all as $id => $path){
			if(!$id){
				continue;
			}
			$data[] = [
				'id' => $id,
				'type' => self::TYPE_TEMPLATE,
				'ct' => we_base_ContentTypes::FOLDER_TEMPLATE,
				'path' => $path,
				'it' => 0
			];
		}
	}

	/**
	 * @param array<int,string> $all
	 * @param array<int,array{id:int,type:string,ct:string,path:string,it:int}> $data
	 * @return void
	 */
	private static function insertTemplatesInArray(array $all, array &$data): void{
		foreach($all as $id => $path){
			$data[] = [
				'id' => (int) $id,
				'type' => self::TYPE_TEMPLATE,
				'ct' => we_base_ContentTypes::TEMPLATE,
				'path' => $path,
				'it' => 0
			];
		}
	}

	/**
	 * Create and returns data Array with IDs and other information for the fragmment class for rebuilding all documents and templates (Called from getDocuments())
	 *
	 * @return array
	 */
	private static function getTemplates(bool $all = false): array{
		if(!($all || we_base_permission::hasPerm([we_base_perm::REBUILD_TEMPLATES]))){
			return [];
		}
		$data = [];
		$db = DB_WE::$inst;
		self::insertTemplateFoldersInArray($db->getAllFirstq('(SELECT ID,Path FROM ' . TEMPLATES_TABLE . ' WHERE ContentType="' . we_base_ContentTypes::FOLDER_TEMPLATE . '" ORDER BY LENGTH(Path))', false), $data); //@phan-suppress-current-line PhanPartialTypeMismatchArgument
		//make a done list with id's
		$done = $db->getAllFirstq('SELECT ID,Path FROM ' . TEMPLATES_TABLE . ' WHERE ContentType="' . we_base_ContentTypes::TEMPLATE . '" AND MasterTemplateID=0 AND IncludedTemplates="" ORDER BY LENGTH(Path)', false) +
			$db->getAllFirstq('SELECT ID,Path FROM ' . TEMPLATES_TABLE . ' WHERE ContentType="' . we_base_ContentTypes::TEMPLATE . '" AND MasterTemplateID IN (SELECT ID FROM ' . TEMPLATES_TABLE . ' WHERE ContentType="' . we_base_ContentTypes::TEMPLATE . '" AND MasterTemplateID=0 AND IncludedTemplates="") AND IncludedTemplates="" ORDER BY LENGTH(Path)', false);
		'@phan-var array<int,string> $done';
//get all easy templates
		self::insertTemplatesInArray($done, $data);
		$doneIds = array_keys($done);
		unset($done);
		//add complex templates
		while(($newDone = $db->getAllFirstq('SELECT t.ID,t.Path
FROM ' .
		TEMPLATES_TABLE . ' t
WHERE
	t.ContentType="' . we_base_ContentTypes::TEMPLATE . '" AND
	t.MasterTemplateID IN(0,' . implode(',', $doneIds) . ') AND
	t.ID NOT IN(' . implode(',', $doneIds) . ') AND
  (SELECT COUNT(1) FROM ' . TEMPLATES_TABLE . ' it WHERE it.ContentType="' . we_base_ContentTypes::TEMPLATE . '" AND FIND_IN_SET(it.ID,t.IncludedTemplates) )=
	(SELECT COUNT(1) FROM ' . TEMPLATES_TABLE . ' it WHERE it.ContentType="' . we_base_ContentTypes::TEMPLATE . '" AND FIND_IN_SET(it.ID,t.IncludedTemplates) AND it.ID IN(' . implode(',', $doneIds) . ') )
ORDER BY LENGTH(t.Path)', false))){
			'@phan-var array<int,string> $newDone';
			self::insertTemplatesInArray($newDone, $data);
			$doneIds = array_merge($doneIds, array_keys($newDone));
		}

//finally add everything still missing
		self::insertTemplatesInArray($db->getAllFirstq('SELECT ID,Path FROM ' . TEMPLATES_TABLE . ' WHERE ContentType="' . we_base_ContentTypes::TEMPLATE . '" AND ID NOT IN (' . implode(',', $doneIds) . ') ORDER BY LENGTH(Path)', false), $data); //@phan-suppress-current-line PhanPartialTypeMismatchArgument
		//self::getDependendTemplates($done, $data);
		return $data;
	}

	private static function getDynamicDocuments(): array{
		$data = [];
		DB_WE::$inst->query('SELECT ID,Path FROM ' . FILE_TABLE . ' WHERE IsFolder=1 ORDER BY LENGTH(Path)');
		while(DB_WE::$inst->next_record()){
			$data[] = [
				'id' => DB_WE::$inst->f('ID'),
				'type' => self::TYPE_DOCUMENT,
				'dyn' => 0,
				'ct' => we_base_ContentTypes::FOLDER_DOCUMENT,
				'mt' => 0,
				'tt' => 0,
				'path' => DB_WE::$inst->f('Path'),
				'it' => 0];
		}

		DB_WE::$inst->query('SELECT ID,ContentType,Path FROM ' . FILE_TABLE . ' WHERE IsDynamic=1 AND IsPublished=1 ORDER BY LENGTH(Path),Filename');

		while(DB_WE::$inst->next_record()){
			$data[] = [
				'id' => DB_WE::$inst->f('ID'),
				'type' => self::TYPE_DOCUMENT,
				'dyn' => 1,
				'ct' => DB_WE::$inst->f('ContentType'),
				'mt' => 0,
				'tt' => 0,
				'path' => DB_WE::$inst->f('Path'),
				'it' => 0];
		}
		return $data;
	}

	/**
	 * Create and returns data Array with IDs and other information for the fragmment class for rebuilding filtered documents (Called from getDocuments())
	 *
	 * @return array
	 * @param int[] $categories csv value of category IDs
	 * @param boolean $catAnd true if AND should be used for more than one categories (default=OR)
	 * @param int[] $doctypes csv value of doctypeIDs
	 * @param int[] $folders csv value of directory IDs
	 * @param int $templateID ID of a template (All documents of this template should be rebuilded)
	 */
	private static function getFilteredDocuments(array $categories, bool $catAnd, array $doctypes, array $folders, int $templateID): array{
		if(!we_base_permission::hasPerm([we_base_perm::REBUILD_FILTERD])){
			return [];
		}
		$data = [];
		$cat_query = $doctype_query = $folders_query = $template_query = '';

		$categories = array_filter($categories);
		if($categories){
			$cat_query = 'CONCAT(",",Category,",") REGEXP ",(' . implode('|', $categories) . '),"';
		}
		if($doctypes){
			$foo = array_filter($doctypes);
			$doctype_query = 'Doctype IN (' . implode(',', $foo) . ')';
		}
		if($folders){
			$foldersList = we_base_file::getFoldersInFolder(array_filter($folders));
			$folders_query = $foldersList ? '( ParentID IN(' . implode(',', $foldersList) . '))' : '';
		}

		if($templateID){
			$arr = self::getTemplAndDocIDsOfTemplate($templateID);

			if($arr['templateIDs']){
				//$done = DB_WE::$inst->getAllq('SELECT ID FROM ' . TEMPLATES_TABLE . ' WHERE IsFolder=0 AND ID NOT IN (' . implode(',', $arr['templateIDs']) . ')', true);
				//self::getDependendTemplates($done, $data);

				$tmp = $arr['templateIDs'];
				$tmp[] = $templateID;
				self::insertTemplatesInArray(id2pathA($tmp, TEMPLATES_TABLE), $data); //@phan-suppress-current-line PhanPartialTypeMismatchArgument
				$template_query = '(TemplateID IN (' . implode(',', $tmp) . '))';
			} else {
				$template_query = '(TemplateID=' . $templateID . ')';
			}
		}

		if(!$categories && !$catAnd && !$doctypes && !$folders && !$templateID){
			DB_WE::$inst->query('SELECT ID,Path FROM ' . FILE_TABLE . ' WHERE IsFolder=1 ' . ($folders_query ? 'AND ' . $folders_query : '') . 'ORDER BY IsFolder DESC, LENGTH(Path)');
			while(DB_WE::$inst->next_record()){
				$data[] = [
					'id' => DB_WE::$inst->f('ID'),
					'type' => self::TYPE_DOCUMENT,
					'dyn' => 0,
					'ct' => we_base_ContentTypes::FOLDER_DOCUMENT,
					'mt' => 0,
					'tt' => 0,
					'path' => DB_WE::$inst->f('Path'),
					'it' => 0];
			}
		}

		DB_WE::$inst->query('SELECT ID,ContentType,Path,IsDynamic FROM ' . FILE_TABLE . ' WHERE IsDynamic=0 AND IsPublished=1 AND ContentType IN("' . we_base_ContentTypes::WEDOCUMENT . '","' . we_base_ContentTypes::CSS . '","' . we_base_ContentTypes::JS . '") ' .
			($cat_query ? ' AND ' . $cat_query . ' ' : '') .
			($doctype_query ? ' AND ' . $doctype_query . ' ' : '') .
			($folders_query ? ' AND ' . $folders_query . ' ' : '') .
			($template_query ? ' AND ' . $template_query . ' ' : '') .
			' ORDER BY IsDynamic DESC,LENGTH(Path)');

		while(DB_WE::$inst->next_record()){
			$data[] = [
				'id' => DB_WE::$inst->f('ID'),
				'type' => self::TYPE_DOCUMENT,
				'dyn' => DB_WE::$inst->f('IsDynamic'),
				'ct' => DB_WE::$inst->f('ContentType'),
				'mt' => 0,
				'tt' => 0,
				'path' => DB_WE::$inst->f('Path'),
				'it' => 0];
		}
		return $data;
	}

	/**
	 * Create and returns data Array with IDs and other information for the fragmment class for rebuilding the OBJECTFILES_TABLE
	 *
	 * @return array
	 */
	public static function getObjects(): array{
		$data = [];
		if(we_base_permission::hasPerm([we_base_perm::REBUILD_OBJECTS])){
			DB_WE::$inst->query('SELECT ID,ContentType,Path FROM ' . OBJECT_FILES_TABLE . ' WHERE IsPublished=1 ORDER BY ID');
			while(DB_WE::$inst->next_record()){
				$data[] = [
					'id' => DB_WE::$inst->f('ID'),
					'type' => self::TYPE_OBJECT,
					'ct' => DB_WE::$inst->f('ContentType'),
					'mt' => 0,
					'tt' => 0,
					'path' => DB_WE::$inst->f('Path'),
					'it' => 0];
			}
			DB_WE::$inst->query('SELECT ID,Path FROM ' . OBJECT_FILES_TABLE . ' WHERE IsFolder=1 ORDER BY ID');
			while(DB_WE::$inst->next_record()){
				$data[] = [
					'id' => DB_WE::$inst->f('ID'),
					'type' => self::TYPE_OBJECT,
					'ct' => we_base_ContentTypes::FOLDER_OBJECTFILE,
					'mt' => 0,
					'tt' => 0,
					'path' => DB_WE::$inst->f('Path'),
					'it' => 0];
			}
		}
		return $data;
	}

	/**
	 * Create and returns data Array with IDs and other information for the fragmment class for rebuilding the INDEX_TABLE
	 *
	 * @return array
	 */
	/* public static function getNavigation(): array{
	  $data = [];
	  if(we_base_permission::hasPerm([we_base_perm::REBUILD_NAVIGATION])){
	  DB_WE::$inst->query('SELECT ID,Path FROM ' . NAVIGATION_TABLE . ' WHERE IsFolder=0 ORDER BY ID');
	  while(DB_WE::$inst->next_record()){
	  $data[] = [
	  'id' => DB_WE::$inst->f('ID'),
	  'type' => 'navigation',
	  'mt' => 0,
	  'tt' => 0,
	  'path' => DB_WE::$inst->f('Path'),
	  'it' => 0];
	  }
	  $data[] = [
	  'id' => 0,
	  'type' => 'navigation',
	  'mt' => 0,
	  'tt' => 0,
	  'path' => DB_WE::$inst->f('Path'),
	  'it' => 0
	  ];
	  }

	  if(we_base_request::bool('rebuildStaticAfterNavi')){
	  $data2 = self::getFilteredDocuments([], false, [], [], 0);
	  $data = array_merge($data, $data2);
	  }

	  return $data;
	  } */

	/**
	 * Create and returns data Array with IDs and other information for the fragmment class for rebuilding the INDEX_TABLE
	 *
	 * @return array
	 */
	public static function getIndex(): array{
		if(!we_base_permission::hasPerm([we_base_perm::REBUILD_INDEX])){
			return [];
		}
		$data = [];
		DB_WE::$inst->query('SELECT ID,ContentType,Path FROM ' . FILE_TABLE . ' WHERE IsPublished=1 AND IsSearchable=1 AND IsFolder=0 ORDER BY ID');
		while(DB_WE::$inst->next_record()){
			$data[] = [
				'id' => DB_WE::$inst->f('ID'),
				'type' => self::TYPE_DOCUMENT,
				'ct' => DB_WE::$inst->f('ContentType'),
				'mt' => 0,
				'tt' => 0,
				'path' => DB_WE::$inst->f('Path'),
				'it' => 1];
		}
		DB_WE::$inst->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=0 AND ID NOT IN (SELECT ID FROM ' . FILE_TABLE . ' WHERE Published>0 AND IsSearchable=1 AND IsFolder=0)');
		/* on mysql 8 and mariadb 10.0.5 this is a VERY fast alternative for "text-files" just need to rebuild binaries
		 * => add self::MAX_INDEX_SIZE, use nHash, remove entities REGEXP_REPLACE(Text,'&#\\d+;',"")
		  REPLACE INTO tblIndex (ID,WorkspaceID,DID,Text,Category,Doctype,Title,Description,Language)
		  SELECT c.DID AS ID,f.ParentID AS WorkspaceID,c.DID AS DID,
		  REGEXP_REPLACE(
		  REGEXP_REPLACE(GROUP_CONCAT(trim(REGEXP_REPLACE(c.Dat, '<.+?>',' '))),'(&#160;|&nbsp;)',' '),
		  "[ \r\n]+",' ') AS Text,
		  f.Category,f.DocType,
		  tit.Dat AS Title,
		  des.Dat AS Description,
		  f.Language

		  FROM
		  tblContent c JOIN tblFile f ON c.DID=f.ID
		  LEFT JOIN tblContent des ON (des.DocumentTable='tblFile' AND c.DID=des.DID AND des.Name="we_contents_root::DESCRIPTION_FIELD")
		  LEFT JOIN tblContent tit ON (tit.DocumentTable='tblFile' AND c.DID=tit.DID AND tit.Name="we_contents_root::TITLE_FIELD")
		  WHERE c.DocumentTable='tblFile' AND c.Dat IS NOT NULL AND c.Type="txt" AND c.Dat NOT REGEXP('^[\\[}$]')
		  AND f.Published>0 AND f.IsSearchable=1 AND f.IsFolder=0 AND f.ContentType IN ('text/html','text/webedition','text/plain')
		  GROUP BY c.DID
		 */


		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			DB_WE::$inst->query('SELECT ID,ContentType,Path FROM ' . OBJECT_FILES_TABLE . ' WHERE IsPublished=1 AND IsFolder=0 ORDER BY TableID,ID');
			while(DB_WE::$inst->next_record()){
				$data[] = [
					'id' => DB_WE::$inst->f('ID'),
					'type' => self::TYPE_OBJECT,
					'ct' => DB_WE::$inst->f('ContentType'),
					'mt' => 0,
					'tt' => 0,
					'path' => DB_WE::$inst->f('Path'),
					'it' => 1];
			}
			DB_WE::$inst->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID>0 AND ID NOT IN (SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE Published>0 AND IsSearchable=1 AND IsFolder=0)');
		} else {
			DB_WE::$inst->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID>0');
		}
		return $data;
	}

	/**
	 * Create and returns data Array with IDs and other information for the fragmment class for rebuilding thumbnails
	 *
	 * @return array
	 * @param int[] $thumbs csv value of IDs which thumbs to create
	 * @param int[] $thumbsFolders csv value of directory IDs => Create Thumbs for images in these directories.
	 */
	public static function getThumbnails(array $thumbs = [], array $thumbsFolders = []): array{
		if(!we_base_permission::hasPerm([we_base_perm::REBUILD_THUMBS])){
			return [];
		}
		$data = [];
		if($thumbsFolders){
			$foldersList = [];
			foreach($thumbsFolders as $folderID){
				$foldersList[] = implode(',', we_base_file::getFoldersInFolder([$folderID]));
			}
			$folders_query = '( ParentID IN(' . implode(',', $foldersList) . ') )';
		} else {
			$folders_query = '';
		}
		DB_WE::$inst->query('SELECT ID,ContentType,Path,Extension FROM ' . FILE_TABLE . ' WHERE ContentType="' . we_base_ContentTypes::IMAGE . '"' . ($folders_query ? ' AND ' . $folders_query : '') . ' ORDER BY ID');
		while(DB_WE::$inst->next_record()){
			$data[] = [
				'id' => DB_WE::$inst->f('ID'),
				'type' => 'thumbnail',
				'ct' => DB_WE::$inst->f('ContentType'),
				'thumbs' => $thumbs,
				'extension' => DB_WE::$inst->f('Extension'),
				'mt' => 0,
				'tt' => 0,
				'path' => DB_WE::$inst->f('Path'),
				'it' => 0];
		}
		return $data;
	}

	/**
	 *
	 * @param int $id
	 * @param int[] $arr
	 * @param bool $onlyMaster
	 * @return void
	 */
	private static function getTemplatesOfTemplate(int $id, array &$arr, bool $onlyMaster = false): void{
		$foo = DB_WE::$inst->getAllq('SELECT ID FROM ' . TEMPLATES_TABLE . ' WHERE
	ContentType="' . we_base_ContentTypes::TEMPLATE . '" AND (
	MasterTemplateID=' . $id . ($onlyMaster ? '' : ' OR FIND_IN_SET(' . $id . ',IncludedTemplates) ') . ')' .
			($arr ? ' AND ID NOT IN (' . implode(',', $arr) . ')' : ''), true);
		'@phan-var int[] $foo';
		if(!$foo){
			return;
		}

		$arr = array_merge($arr, $foo);
		if(in_array($id, $arr)){
			return;
		}

		foreach($foo as $check){
			self::getTemplatesOfTemplate($check, $arr);
		}
	}

	static function getTemplAndDocIDsOfTemplate(int $id, bool $staticOnly = true, bool $publishedOnly = false, bool $PublishedAndTemp = false, bool $useLockTbl = false): array{
		$returnIDs = [
			'templateIDs' => [],
			'documentIDs' => [],
		];

		if(!$id){
			return $returnIDs;
		}

		self::getTemplatesOfTemplate($id, $returnIDs['templateIDs'], $publishedOnly);

// Bug Fix 6615
		$tmpArray = $returnIDs['templateIDs'];
		$tmpArray[] = $id;
		$tmp = implode(',', array_filter($tmpArray));
		unset($tmpArray);
		if($useLockTbl){
			$returnIDs['templateIDs'] = DB_WE::$inst->getAllq('SELECT ID FROM ' . LOCK_TABLE . ' WHERE tbl="' . stripTblPrefix(TEMPLATES_TABLE) . '" AND ID IN (' . $tmp . ')', true);
		}
		$where = ' (' .
			($PublishedAndTemp ? 'temp_template_id IN (' . $tmp . ') OR ' : '') .
			' TemplateID IN (' . $tmp . ')' .
			')' .
			($staticOnly ? ' AND IsDynamic=0' : '') .
			($publishedOnly ? ' AND IsPublished=1' : '');

		$returnIDs['documentIDs'] = DB_WE::$inst->getAllq('SELECT f.ID FROM ' . FILE_TABLE . ' f ' . ($useLockTbl ? 'INNER JOIN ' . LOCK_TABLE . ' l ON f.ID=l.ID AND l.tbl="' . stripTblPrefix(FILE_TABLE) . '"' : '') . ' WHERE ' . $where, true);

		return $returnIDs;
	}

}
